using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Render_Modes(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {
            /* Sample: Render_Modes
	         * This sample will show you how to:
	         * - Load an HSF file through Stream and check for Stream IO exceptions
	         * - Divide the screen into sub-window
	         * - Include a segment into other segments
	         * - Render with flat shading, wireframe, hidden line, gouraud and phong algorithms
	         * - Use color index interpolation
	         * 
	         * Read more about the topics covered in this sample in our Programming Guide 
	         * - section 7.5:	Lighting Interpolation*/

            /* This next block of code shows you how to load an HSF file
             * 1. Create an ImportOptionsKit and set the segment in which you want the file to be loaded
             * 2. Use the Stream::File::Import method to load the desired file, with the options you specified
             * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding 
             * 4. Use a try-catch block to capture any Stream IO exceptions 
             * 5. Root segments are special segments, which can only be created from the Database. Since this root segment
             *		is not associated with a window, its contents will not be displayed on the screen. This segment
             *		will be used to store geometry which can later be included in segments which are under a window */
            HPS.SegmentKey includeLibrary = HPS.Database.CreateRootSegment();
            HPS.SegmentKey urn = includeLibrary.Subsegment();

	        //Loading an HSF file into a specific segment
            HPS.Stream.ImportNotifier notifier;
            HPS.SegmentKey phongKey = modelKey.Subsegment();
	        try
	        {

                HPS.Stream.ImportOptionsKit importOptionsKit = new HPS.Stream.ImportOptionsKit();
		        importOptionsKit.SetSegment(urn);
		        string filename = INPUT_FOLDER + "/urn.hsf";

                notifier = HPS.Stream.File.Import(filename, importOptionsKit);
		        notifier.Wait();
	        }
            catch (HPS.IOException)
	        {
		        /* catch Stream I/O Errors here */
	        }

	        urn.GetModellingMatrixControl().Rotate(-25, 0, 0);

            HPS.TextAttributeKit textAttributeKit = new HPS.TextAttributeKit();
            HPS.VisibilityKit visibilityKit = new HPS.VisibilityKit();
            HPS.CameraKit cameraKit = new HPS.CameraKit();

	        //Setting up the model segment with visibility and text attributes
	        //Creating a local camera.
            textAttributeKit.SetAlignment(HPS.Text.Alignment.TopCenter);
            textAttributeKit.SetSize(0.03f, HPS.Text.SizeUnits.WindowRelative);

	        modelKey.SetTextAttributes(textAttributeKit);
            modelKey.GetMaterialMappingControl().SetFaceColor(new HPS.RGBAColor(1, 0, 0, 1));

            HPS.SubwindowKit subwindowKit = new HPS.SubwindowKit();

            /* This block of code divides the screen into subsegments, includes geometry in the scene
             * and sets different rendering options on each segment
             * 1. To set a sub-window call the SetSubwindow method with the dimensions of the sub-window, 
             *		normalized between -1 and 1, with (-1, 1) pointing to the top left corner
             * 2. To include a segment into another segment use the following method:
             *		destinationSegmentKey.IncludeSegment(sourceSegmentKey)
             *		including segments is a good way to add the same geometry in multiple places without
             *		needlessly creating multiple copies of the geometry in question 
             * 3. Change the lighting interpolation algorithm used for a segment by using the 
             *		LightingAttributeControl::SetInterpolationAlgorithm() method
             * 4. Segment attributes are changed as needed through the use of kits and controls
             * 
             * NOTE: Lights do not shine through sub-windows, therefore it is necessary to insert a new light 
             *		per sub-window to see the results of the lighting algorithms*/
            HPS.SegmentKey wireframeKey = modelKey.Subsegment();
	        subwindowKit.SetSubwindow(new HPS.Rectangle(-1, -0.33f, 0, 1));
	        wireframeKey.GetVisibilityControl().SetFaces(false).SetEdges(true).SetLights(false);

	        wireframeKey.IncludeSegment(urn);

	        wireframeKey.SetSubwindow(subwindowKit);
	        wireframeKey.SetVisibility(visibilityKit);
            wireframeKey.InsertText(new HPS.Point(0, 1.40f, 0), "Wireframe");

	        //Index Interpolated rendering
	        //TODO - color index interpolation not yet supported
            HPS.SegmentKey indexInterpolatedKey = modelKey.Subsegment();
	        subwindowKit.SetSubwindow(new HPS.Rectangle(-0.33f, 0.33f, 0, 1));
	        indexInterpolatedKey.IncludeSegment(urn);
	        indexInterpolatedKey.SetSubwindow(subwindowKit);
            indexInterpolatedKey.InsertText(new HPS.Point(0, 1.40f, 0), "Color Index Interpolated");

	        //Gouraud Shaded
            HPS.SegmentKey gouraudKey = modelKey.Subsegment();
            gouraudKey.InsertDistantLight(new HPS.Vector(1, 1, 0));
            gouraudKey.GetLightingAttributeControl().SetInterpolationAlgorithm(HPS.Lighting.InterpolationAlgorithm.Gouraud);

	        subwindowKit.SetSubwindow(new HPS.Rectangle(0.33f, 1, 0, 1));

	        gouraudKey.IncludeSegment(urn);
	        gouraudKey.SetSubwindow(subwindowKit);
	        gouraudKey.GetVisibilityControl().SetLights(true);

            gouraudKey.InsertText(new HPS.Point(0, 1.40f, 0), "Gouraud");

	        //Hidden Surface and Lines
            HPS.SegmentKey hiddenSurfaceKey = modelKey.Subsegment();
	        subwindowKit.SetSubwindow(new HPS.Rectangle(-1, -0.33f, -1, 0));
	        visibilityKit.UnsetEverything().SetFaces(true).SetEdges(true).SetLights(false);

	        hiddenSurfaceKey.IncludeSegment(urn);

	        hiddenSurfaceKey.SetSubwindow(subwindowKit);
	        hiddenSurfaceKey.SetVisibility(visibilityKit);
            hiddenSurfaceKey.GetSubwindowControl().SetRenderingAlgorithm(HPS.Subwindow.RenderingAlgorithm.HiddenLine);
	        hiddenSurfaceKey.GetHiddenLineAttributeControl().SetVisibility(false).SetLinePattern("-  -  -")
		        .SetDimFactor(0.5f).SetFaceDisplacement(0.5f);
            hiddenSurfaceKey.InsertText(new HPS.Point(0, 1.40f, 0), "True Hidden Line");

	        //Flat Shaded
            HPS.SegmentKey flatKey = modelKey.Subsegment();
            flatKey.InsertDistantLight(new HPS.Vector(1, 1, 0));
            flatKey.GetLightingAttributeControl().SetInterpolationAlgorithm(HPS.Lighting.InterpolationAlgorithm.Flat);
	        subwindowKit.SetSubwindow(new HPS.Rectangle(-0.33f, 0.33f, -1, 0));

	        flatKey.IncludeSegment(urn);
	        flatKey.SetSubwindow(subwindowKit);
	        flatKey.InsertText(new Point(0, 1.40f, 0), "Flat Shaded");

	        //Phong Shaded
            //Currently not working properly. Eventum ticket# 19265
            phongKey.InsertDistantLight(new HPS.Vector(1, 1, 0));
            phongKey.GetLightingAttributeControl().SetInterpolationAlgorithm(HPS.Lighting.InterpolationAlgorithm.Phong);
	        phongKey.GetVisibilityControl().SetLights(true);
	        subwindowKit.SetSubwindow(new HPS.Rectangle(0.33f, 1, -1, 0));

	        phongKey.IncludeSegment(urn);
	        phongKey.SetSubwindow(subwindowKit);
            phongKey.InsertText(new HPS.Point(0, 1.40f, 0), "Phong");
            view.FitWorld().Update();
        }
    }
}
